function [flag,x,y,C,E]=at(A,b,c,d) % directly uses verinfeas, veropt (twice), verunbound, verdual; indirectly lpprocedure, versol, optsolset; suppressed: verpurify
%    AT       Verified linear programming.  
%
%    NOTE to be deleted in the final version: input variable d should be suppressed.
%
%    This is an INTLAB file. It requires to have INTLAB installed under
%    MATLAB to function properly.
%
%    For a real matrix A (full or sparse) and matching real vectors b, c,
%        [flag,x,y,C,E]=at(A,b,c)
%    either computes verified optimal solution x, verified dual optimal solution y
%    and verified optimal value h of the linear programming problem
%        min c'*x   subject to   A*x=b, x>=0,
%    or verifies (in)feasibility, or verifies unboundedness, or yields no
%    verified result. The respective outcome is always described verbally
%    in the variable flag.
%
%    Possible outputs:
%
%    flag='verified optimum   ' : x is verified to enclose a primal optimal solution, 
%                                 y is verified to enclose a dual optimal solution, 
%                                 C.h is verified to enclose the optimal value,
%                                 C.B is the set of indices of the basic variables,
%                                 C.N is the set of indices of the nonbasic variables,
%                                 C.crit is the final criterial row,
%    flag='verified unbounded ' : x is verified to enclose a primal feasible solution xo, and
%                                 y is verified to enclose a vector yo such that the objective 
%                                   tends to -Inf along the feasible half-line {xo+t*yo | t>=0},
%                                 C fields consist of NaN's,
%    flag='verified feasible  ' : x is verified to enclose a primal feasible solution
%                                   (optimality nor unboundedness could be verified),
%                                 y and C are of NaN's,
%    flag='verified infeasible' : y is verified to enclose a Farkas vector yo satisfying A'*yo>=0, b'*yo<0
%                                   (whose existence proves primal infeasibility),
%                                 x and C are of NaN's,
%    flag='no verified result ' : x, y and C are all of NaN's
%                                   (no verified result could be found),
%    flag='sizes do not match ' : x, y and C are all of NaN's
%                                   (sizes of A, b, c do not match).
%
%    The structured array E explains reasons for NaN output. It has three fields: 
%    E.error, E.where, E.value. 
%
%    See also LINPROG, VERIFYLSS.

%    Copyright 2007-2008 Jiri Rohn
%
%    A part of this work (nondegenerate solutions) was supported by the
%    Czech Republic National Research Program "Information Society",
%    project 1ET400300415. The second part (degenerate solutions) was done
%    during author's employment at the Anglo-American University in Prague,
%    Czech Republic. 
%
%    WARRANTY
%
%    Because the program is licensed free of charge, there is 
%    no warranty for the program, to the extent permitted by applicable
%    law. Except when otherwise stated in writing the copyright holder
%    and/or other parties provide the program "as is" without warranty
%    of any kind, either expressed or implied, including, but not
%    limited to, the implied warranties of merchantability and fitness
%    for a particular purpose. The entire risk as to the quality and
%    performance of the program is with you. Should the program prove
%    defective, you assume the cost of all necessary servicing, repair
%    or correction.
%
%    History
%
%    2007, Spring first working version created
%    2007-05-31   first posted as VERLINPROG; handles nondegenerate solutions only
%    2008-09-10   started again
%    2008-09-11   degenerate solutions handled via new VEROPT using VERSOL
%    2008-10-08   output structured array C created, Xopt added into it (subfunction optsolset)
%    2008-10-11   optsolset included into main file
%    2008-11-08   output of C.Xopt suppressed
%    2008-11-09   C's fields explained in the help
%    2008-11-28   input variable d and all the messages added
%
gr=getround;
setround(0);
b=b(:); c=c(:); [m,n]=size(A); p=length(b); q=length(c);
% defaults
flag='no verified result ';
x=repmat(infsup(NaN,NaN),n,1); 
y=repmat(infsup(NaN,NaN),m,1); 
C.h=infsup(NaN,NaN);
C.B=repmat(NaN,1,m);
C.N=repmat(NaN,1,n-m);
% C.BBB=C.B; % output suppressed
% C.NNN=C.N; % output suppressed
C.crit=repmat(infsup(NaN,NaN),1,n);
% C.Xopt=repmat(infsup(NaN,NaN),m,n-m); % presently suppressed
E.error='at: none';
E.where='NaN';
E.value='NaN';
if ~(m==p&&n==q)||(m>n), 
    flag='sizes do not match ';
    E.error='at: sizes do not match ';
    setround(gr); return
end
if isintval(A)||isintval(b)||isintval(c),                       % error('data not real')
    E.error='at: data not real';
    setround(gr); return
end
if ~(nargin==4&&isequal(d,1))
    d=0;
end
if issparse(b)
    b=full(b);
end
if issparse(c)
    c=full(c);
end
% [A,b]=verpurify(A,b); % purifying of dependent equations % deleted (dual output in another dimensions; computation of pseudoinverse etc.)
% verifying infeasibility
if d==1
    disp('verifying infeasibility ...')
end
if d==1
    disp('   inferring infeasibility from the data ...')
end
for i=1:m
    if (all(A(i,:)>=0)&&b(i)<0) % i-th equation cannot have nonnegative solution
        y=zeros(m,1); y(i)=1; flag='verified infeasible';
        if d==1
            disp('   infeasibility verified ...')
        end
        setround(gr); return
    end
    if (all(A(i,:)<=0)&&b(i)>0) % i-th equation cannot have nonnegative solution
        y=zeros(m,1); y(i)=-1; flag='verified infeasible';
        if d==1
            disp('   infeasibility verified ...')
        end
        setround(gr); return
    end
end
if d==1
    disp('   no result ...')
end
yi=verinfeas(A,b,d);
if ~isnan(yi.inf(1))                                            % verified Farkas vector found
    y=yi; flag='verified infeasible';
    setround(gr); return
end
% verifying feasibility
if d==1
    disp('verifying feasibility ...')
end
[xf,B1,N1,Everopt]=veropt(A,b,ones(n,1),d);
if isnan(xf.inf(1))                                             % verified feasible solution not found
    flag='no verified result ';
    E=Everopt;
    E.error=['at:' E.error];
    setround(gr); return
end
% verifying unboundedness
if d==1
    disp('verifying unboundedness ...')
end
if ~all(c>=0) % c(i)<0 for some i
    yu=verunbound(A,c,d);
    if ~isnan(yu.inf(1))                                            % verified descent direction found
        x=xf; y=yu; flag='verified unbounded ';
        setround(gr); return
    end
else % c>=0
    if d==1
        disp('   inferring impossibility of unboundedness from the data ...')
        disp('   unboundedness excluded ...')

    end
end
% verifying optimality
if d==1
    disp('finding primal optimal solution ...')
end
[xo,B,N,Everopt]=veropt(A,b,c,d);
if isnan(xo.inf(1))                                             % verified feasible primal solution with basis B not found
    x=xf; flag='verified feasible  ';                           % previous feasible solution outputted
    E=Everopt;
    E.error=['at:' E.error];
    setround(gr); return
end
% finding dual optimal solution
if d==1
    disp('finding dual optimal solution ...')
end
[yy,BBB,NNN,ccrit,Xopt,Everdual]=verdual(A,c,B,N,d); 
if isnan(yy.inf(1))                                             % verified feasible dual solution not found
    x=xo; flag='verified feasible  ';                           % candidate for optimum outputted as feasible solution
    E=Everdual;
    E.error=['at:' E.error];
    setround(gr); return
end   
% verified quantities                                           % verified primal and dual feasible solutions found
x=xo;                                                           % x is a verified primal optimal solution
y=yy;                                                           % y is a verified dual optimal solution
h1=c'*x; h2=b'*y;                                               
h=intersect(h1,h2);                                             % h is a verified optimal value (duality theorem) 
if isnan(h.inf)
    h=h1;
end
flag='verified optimum   ';
% additional results put into the output variable C
C.h=h;
C.B=B'; C.B=sort(C.B); % made sorted row vector
C.N=N'; C.N=sort(C.N); % made sorted row vector
% C.BBB=BBB; % output suppressed
% C.NNN=NNN; % output suppressed
C.crit=ccrit;
% C.Xopt=Xopt; % constructed but not outputted; % presently suppressed
setround(gr); 
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Subfunctions: veropt, lpprocedure, versol, verdual, optsolset, verinfeas, verunbound, verpurify
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
function [x,B,N,E]=veropt(A,b,c,d) % uses lpprocedure, versol (twice)
% B is the "basis index set" of an optimal solution of the LP problem
% min c'*x  subject to  A*x=b, x>=0,
% x is a verified basic feasible solution with this basis
% N is the set of nonbasic indices
% E is the error variable
%
% History
%
% 2007, Spring created as a part of VERLINPROG
% 2008-09-11   updated for degenerate solutions (using VERSOL)
% 2008-11-28   input variable d added
%
b=b(:);
[m,n]=size(A); 
x=repmat(infsup(NaN,NaN),n,1);
% B=repmat(NaN,m,1); N=repmat(NaN,n,1);
% BB=repmat(NaN,m,1);
E.error='veropt: none';
E.where='NaN';
E.value='NaN';
if d==1
    disp('   solving unverified linear programming problem ...')
end
xopt=lpprocedure(A,b,c);
[xx,J]=sort(xopt); B=J(n-m+1:n); N=J(1:n-m);                    % B is set of "basic" indices, N of "nonbasic" ones
% this is the main weak point: transformation of a nonbasic solution to a basic one
% xB=verifylss(A(:,B),b);                                       % old version using full A; replaced by next 8 lines
AB=A(:,B);
if issparse(AB)
    AB=full(AB);                                                % only the square submatrix taken full (because of VERIFYLSS)
end
if d==1
    disp('   finding basic solution via verifylss ...')
end
xB=verifylss(AB,b); % requires full AB % possibly redundant (versol also uses verifylss), but left as it is
if isnan(xB.inf(1))
    if d==1
        disp('   basic solution not found ...')
        disp('   finding basic solution via versol ...')
    end
    [xB,Eversol]=versol(AB,b,d);
end
if isnan(xB.inf(1))                                             
    E=Eversol;
    if d==1
        disp('   basic solution not found ...')
    end
    return                                                      % verified "optimal" solution not found 
end
% xB found
if any(xB.sup<0)
    E.error='veropt:verifylss,versol: at least one variable verified negative: improper basis found';
    E.where=B;
    E.value=xB;
    if d==1
        disp('   basic solution not found ...')
    end
    return
end
% xB.sup>=0
xx=infsup(zeros(n,1),zeros(n,1));
xx(B)=xB; % full vector
% xx.sup>=0
if all(xx.inf>=0)                                               % B-part nonnegative, verified "optimal" solution found
    x=xx;
    if d==1
        disp('   basic solution found ...')
    end
    return
end
BB=find(xx.inf>0); % important: not BB=find(xx.inf>=0)          % indices of current positive entries
% for j not in BB, xx.sup(j)>=0 and xx.inf(j)<=0
if isempty(BB) % xx.sup>=0, xx.inf<=0
    if isequal(b,zeros(m,1)) % equation has the form A*x=0
        x=infsup(zeros(n,1),zeros(n,1));
        if d==1
            disp('   basic solution found ...')
        end
        return
    else % b~=0
        E.error='veropt: subbasis empty';
        E.where=BB;
        E.value=[];
        if d==1
            disp('   basic solution not found ...')
        end
        return
    end
end
% BB nonempty; for j in BB, xx.inf(j)>0; for j not in BB, xx.sup(j)>=0 and xx.inf(j)<=0
ABB=A(:,BB);
if issparse(ABB)
    ABB=full(ABB); % versol requires full matrix (error disc'd 2008-10-10)
end
if d==1
    disp('   handling degenerate solution via versol ...')
end
[xBB,Eversol]=versol(ABB,b,d); % solving A(:,BB)*xBB=b (for j not in BB, x(j) set to zero)
if isnan(xBB.inf(1)) % verified "optimal" solution not found
    E=Eversol;
    E.error=['veropt:' E.error];
    if d==1
        disp('   basic solution not found ...')
    end
    return
end
% xBB found
if ~all(xBB.inf>=0) % verified "optimal" solution found but not nonnegative
    E.error='veropt:versol: solution not nonnegative';
    E.where=BB;
    E.value=xBB;
    if d==1
        disp('   basic solution not found ...')
    end
    return
end
% xBB.inf>=0                                            
x=infsup(zeros(n,1),zeros(n,1));
x(BB)=xBB; % full vector
if d==1
        disp('   basic solution found ...')
end
% end of veropt
%
function x=lpprocedure(A,b,c) % presently uses linprog % d not necessary
% solves linear programming problem min c'*x subject to A*x=b, x>=0
% x should be always assigned (unverified optimal solution, or something else; the result is checked afterwards)
% placed separately so that a different linear programming procedure might also be used
n=length(c);
wg=warning;                
warning off
options=optimset('linprog');                                  
options=optimset(options,'Display','off'); % suppresses displaying linprog's error messages
x=linprog(c,-eye(n),zeros(n,1),A,b,[],[],[],options);
warning(wg);
% end of lpprocedure
%
function [x,E]=versol(A,b,d) % uses verfullcolrank (twice), verpinv
%    VERLSOL       Verified solution of a system of linear equations.   
%
%    This is an INTLAB file. It requires to have INTLAB installed under
%    MATLAB to function properly.
%
%    For an m-by-n real matrix A and a matching real vector b,
%        [x,E]=versol(A,b,d)
%    either yields a verified solution x of the system of linear equations A*x=b 
%    which has at least n-r zero entries, where r is the rank of A ("basic solution"),
%    or fails (x is a vector of NaN's).
%
%    See also VERLINSYS.

%    Copyright 2008 Jiri Rohn
%
%    WARRANTY
%
%    Because the program is licensed free of charge, there is 
%    no warranty for the program, to the extent permitted by applicable
%    law. Except when otherwise stated in writing the copyright holder
%    and/or other parties provide the program "as is" without warranty
%    of any kind, either expressed or implied, including, but not
%    limited to, the implied warranties of merchantability and fitness
%    for a particular purpose. The entire risk as to the quality and
%    performance of the program is with you. Should the program prove
%    defective, you assume the cost of all necessary servicing, repair
%    or correction.
%
%    History
%
%    2008, Spring created as a part of VERLINSYS
%    2008-09-06   extracted from VERLINSYS and simplified
%    2008-11-28   input variable d added
%
gr=getround;
setround(0);
b=b(:); 
[m,n]=size(A); 
% defaults
x=repmat(infsup(NaN,NaN),n,1);
E.error='versol: none';
E.where='NaN';
E.value='NaN';
% data check
if nargin~=3||nargout>2||m~=length(b)||~isreal(A)||~isreal(b)||isintval(A)||isintval(b)
    E.error='versol: wrong data';
    setround(gr); return
end
% square case
if m==n
    if d==1
        disp('      solving the square case via verifylss ...')
    end
    xx=verifylss(A,b);
    if ~isnan(xx.inf(1)) % computed
        x=xx;
        if d==1
            disp('      solution found ...')
        end
        setround(gr); return
    end
end
% general case
if d==1
    disp('      computing RREF ...')
end
[AR,K]=rref(A); % K index set of the expected basis (later BS)
if isempty(K) % no basis index set found
    if isequal(A,zeros(m,n))&&isequal(b,zeros(m,1)) % equation O*x=0
        x=zeros(n,1);
        if d==1
            disp('      solution found ...')
        end
        setround(gr); return
    else
        E.error='versol: no basis index set found';
        if d==1
            disp('      solution not found ...')
        end
        setround(gr); return
    end
end
% K nonempty
BS=A(:,K); % expected basis
if d==1
    disp('      using verfullcolrank for checking linear independence of columns of expected basis ...')
end
fcr=verfullcolrank(BS);
if fcr~=1 
    E.error='versol:verfullcolrank: columns of expected basis not verified linearly independent';
    E.where=BS;
    E.value=fcr;
    if d==1
        disp('      solution not found ...')
    end
    setround(gr); return
end
% columns of BS verified linearly independent
if d==1
    disp('      using verfullcolrank for checking linear independence of columns of augmented basis ...')
end
fcrBSb=verfullcolrank([BS b]); 
if fcrBSb~=0
    E.error='versol:verfullcolrank: columns of augmented matrix not verified linearly dependent';
    E.where=[BS b];
    E.value=fcrBSb;
    if d==1
        disp('      solution not found ...')
    end
    setround(gr); return    
end
% existence of solution verified
if d==1
    disp('      computing verified pseudoinverse of the basis ...')
end
XBS=verpinv(BS); % used for computation of x
if isnan(XBS.inf(1,1))
    E.error='versol:verpinv: verified pseudoinverse not computed';
    if d==1
        disp('      solution not found ...')
    end
    setround(gr); return
end
if d==1
    disp('      computing verified solution ...')
end
xK=XBS*b; % xK verified to enclose a solution of BS*X=b
x=infsup(zeros(n,1),zeros(n,1)); x(K)=xK; % x verified to enclose a solution of A*X=b
if d==1
    disp('      solution found ...')
end
setround(gr);
% end of versol
%
function [y,BBB,NNN,ccrit,Xopt,E]=verdual(A,c,B,N,d) % uses versol (twice), optsolset
% computes dual solution for given B, criterial row ccrit and 
% the set of optimal solutions Xopt
%
% History
%
% 2008-09-11   first version
% 2008-10-08   set Xopt of optimal solutions added, BBB and NNN outputted
% 2008-11-28   input variable d added
%
c=c(:);
[m,n]=size(A);
y=repmat(infsup(NaN,NaN),m,1);
BBB= repmat(infsup(NaN,NaN),1,m);
NNN= repmat(infsup(NaN,NaN),1,n-m);
ccrit=repmat(infsup(NaN,NaN),1,n);
Xopt=repmat(infsup(NaN,NaN),m,n-m);
E.error='verdual: none';
E.where='NaN';
E.value='NaN';
AB=A(:,B);
if issparse(AB)
    AB=full(AB);                                                % only the square submatrix taken full
end
if d==1
    disp('   computing optimal dual solution via verifylss ...')
end
yB=verifylss(AB',c(B)); % requires full AB
if isnan(yB.inf(1))
    if d==1
        disp('   solution not found ...')
        disp('   computing dual optimal solution via versol ...')
    end
    [yB,Eversol]=versol(AB',c(B),d);
end
if isnan(yB.inf(1))                                             
    E=Eversol;
    E.error=['verdual:' E.error];
    if d==1
        disp('   optimal dual solution not found ...')
    end
    return                                                      % verified "optimal" solution not found 
end
% yB found
crit=c'-yB'*A;                                                  % criterial row (dual feasibility); should be nonnegative
if all(crit.inf(N)>0) % added 2008-11-28: dual nondegenerate solution found
    y=yB;
    if d==1
        disp('   optimal dual solution found ...')
    end
    return
end
% any(crit.inf(N)<=0)
if any(crit.sup<0)
    E.error='verdual:verifylss,versol: at least one crit(j) verified negative';
    E.where=1:n;
    E.value=crit;
    if d==1
        disp('   optimal dual solution not found ...')
    end
    return    
end
% crit.sup>=0
BBB=find(crit.inf<=0); % includes B
% for j in BBB, crit.inf(j)<=0<=crit.sup(j)
% for j not in BBB, crit.inf(j)>0
% BBB nonempty (includes B and at least one index from N)
if d==1
    disp('   computing dual optimal solution via versol in case of dual degeneracy ...')
end
[yBBB,Eversol]=versol(A(:,BBB)',c(BBB),d);  % solving yBBB'*A(:,BBB)=cBBB' 
if isnan(yBBB.inf(1))   % verified "optimal" solution not found
    E=Eversol;
    E.error=['verdual:' E.error];
    if d==1
        disp('   optimal dual solution not found ...')
    end
    return
end
% yBBB found
NNN=[];
for j=1:n
    if all(BBB~=j)
        NNN=[NNN j];
    end
end
% NNN=1:n - BBB
crit=c'-yBBB'*A; % new criterial row
critNNN=crit(NNN); % NNN-part of it
% if ~all(critNNN.inf>=0)    % verified "optimal" solution found but not nonnegative
if ~all(critNNN.inf>0) % verified "optimal" dual solution found but NNN-part not positive
    E.error='verdual:versol: positivity of the NNN-part of criterial row not established';
    E.where=NNN;
    E.value=critNNN;
    if d==1
        disp('   optimal dual solution not found ...')
    end
    return
end
% critNNN.inf>0                                            
y=yBBB; % y outputted, but Xopt can be of NaN's (exception of the usual rule in order not to destroy an optimal solution found)
ccrit=crit;
ccrit(:,BBB)=repmat(intval(0),1,length(BBB)); % ccrit outputted with zero BBB part
if d==1
    disp('   optimal dual solution found ...')
end
% description of the set of optimal solutions % critNNN.inf>0
if length(BBB)>length(B) % in each opt. sol. x, it must be x(j)==0 for each j in NNN (complementary slackness conds.)
    if d==1
        disp('   describing the set of optimal solutions (output suppressed) ...')
    end
    [Xopt,Eoptsolset]=optsolset(A,B,AB,BBB);
    if isnan(Xopt.inf(1,1))
        E=Eoptsolset;
        E.error=['verdual:' E.error];
    end
else % length(BBB)==length(B), BB==B, critNNN.inf>0, uniqueness of optimal solution established
    Xopt=repmat(infsup(0,0),m,n-m);
end
% end of verdual
%
function [Xopt,E]=optsolset(A,B,AB,BBB) % uses inv only % 2008-11-08: used, but final output of Xopt suppressed
% description of the set of optimal solutions
% length(BBB)>length(B) (B proper part of BBB)
% 2008-10-08 created
% 2008-10-10 error in construction of BBBmB repaired
% 2008-10-11 included into AT
[m,n]=size(A);
E.error='optsolset: none';
E.where='NaN';
E.value='NaN';
ABm1=inv(infsup(AB,AB));
if isnan(ABm1.inf(1,1)) % inverse of AB not computed
    Xopt=repmat(infsup(NaN,NaN),m,n-m);
    E.error='optsolset: inverse of the basis matrix not computed';
    E.where=infsup(AB,AB);
    E.value=ABm1;
    return % with Xopt of NaN's
end
% inverse of AB computed
X=repmat(intval(0),n,length(BBB)-length(B)); % n instead of m; corrected 2008-11-29
BBBmB=[];
for j=1:length(BBB) % computing the set difference BBB-B
    if all(BBB(j)~=B)
        BBBmB=[BBBmB j];
    end
end
% BBBmB=BBB-B is nonempty because of length(BBB)>length(B) (B proper part of BBB)
X(B,:)=-ABm1*A(:,BBBmB);
X(BBBmB,:)=eye(length(BBBmB),length(BBBmB));
% X(NNN,:)=zeros(...)
Xopt=X; % the set of optimal solutions is described as { x-Xopt*t | t>=0 } where x is the optimal solution outputted by the main file
% end of optsolset
%
function [y,E]=verinfeas(A,b,d) % uses veropt
% y verified to enclose a Farkas vector yo (i.e., satisfying A'*yo>=0, b'*yo<0)
% its existence implies infeasibility of A*x=b
[m,n]=size(A);
y=repmat(infsup(NaN,NaN),m,1);
E.error='verinfeas: none';
E.where='NaN';
E.value='NaN';
ep=max(1e-08,max([m n 100])*max([norm(A,inf) norm(b,inf)])*eps);
Afv=[A' -A'    -eye(n) zeros(n,1);                              % Afv is (n+1)x(2*m+n+1)
     b' -b' zeros(1,n)         1]; 
bfv=[zeros(n,1)' -1]';                                          % bfv is (n+1)x1
bfv=bfv+ep*[ones(1,n) -1]';                                     % perturbation to compensate roundoff errors (so that A'*y>=0)
[yf,B1,N1,Everopt]=veropt(Afv,bfv,ones(2*m+n+1,1),d);           % system: A'*y>=0, b'*y<=-1, y written as y=y1-y2
if ~isnan(yf.inf(1))
    yf=mid(yf);
    y1=yf(1:m);
    y2=yf(m+1:2*m);
    yf=y1-y2;                                                   % would-be Farkas vector
    A=infsup(A,A); b=infsup(b,b); yf=infsup(yf,yf);             % (i.e., should satisfy A'*y>=0, b'*y<0)
    alpha=A'*yf; beta=b'*yf;
    if (all(alpha.inf>=0))&&(beta.sup<0)                        % infeasibility verified
        y=yf;                                                   % Farkas vector outputted
        return
    else
        E.error='verinfeas: vector found does not have the Farkas property';
        E.where=yf;
        E.value=[alpha.inf; beta.sup];
        return
    end
else
    E=Everopt;
    E.error=['verinfeas:' E.error];
    return
end
% end of verinfeas
%
function [y,E]=verunbound(A,c,d) % uses veropt
% y verified to enclose a vector yo satisfying A*yo=0, yo>=0, c'*yo<=-1
% under feasibility its existence implies unboundedness
[m,n]=size(A);
y=repmat(infsup(NaN,NaN),n,1);
E.error='verunbound: none';
E.where='NaN';
E.value='NaN';
Aunb=[A zeros(m,1);                                             % Aunb is (m+1)x(n+1)
      c'        1];
bunb=[zeros(1,m) -1]';                                          % bunb is (m+1)x1
[yunb,B1,N1,Everopt]=veropt(Aunb,bunb,ones(n+1,1),d);           % yunb is (n+1)x1
if ~isnan(yunb.inf(1))
    y=yunb(1:n);                                                % y satisfies A*y=0, y>=0, c'*y=-1 
    return
else
    E=Everopt;
    E.error=['verunbound:' E.error];
end
% end of verunbound
%
function [AA,bb]=verpurify(A,b) % presently not used; prolongs computation, mostly unnecessarily
% purifies A*x=b of dependent equations
% A*x=b and AA*x=bb have the same solution set, AA*x=bb possibly less rows
% 2008-09-15
[m n]=size(A);
B=verbasis([A b]');
if ~isempty(B)&&~isnan(B(1,1))&&size(B,2)<m % dependent equations found
    B=B'; % B has n+1 columns and less than m rows
    AA=B(:,1:n);
    bb=B(:,n+1);
else % dependent equations not found
    AA=A;
    bb=b;
end
% end of verpurify

