function [x,y,C]=ek(A,B,b)
%    EK    Solving the equation A*x+B*abs(x)=b.
%
%    [x,y,C]=ek(A,B,b) either finds a solution x of the equation  
%        A*x+B*abs(x)=b 
%    (A, B square), or finds a nonzero vector y satisfying
%        abs(A*y)<=abs(B)*abs(y).
%    In the latter case it also finds a singular matrix C.As such 
%    that C.As*y=0 and  
%        abs(C.As-A)<=abs(B). 
%    At most one of the output variables x, y is assigned. The variable
%    C.flag gives a verbal description of the output ('solution' or
%    'singular'), C.iter is the number of iterations.   
%
%    The computation is performed in pure MATLAB (not in INTLAB), so that
%    the results are not verified.

%    Copyright 2003-2008 Jiri Rohn 
%
%    History   
%
%    2005-11-17   absvaleqn, final version
%    2007-12-12   output changed (x, As, NaN's); evakop
%    2008-04-24   output changed (x, y, C); ek
%
gr=getround;
setround(0);
b=b(:); 
[m,n]=size(A);
x=repmat(NaN,n,1); 
y=repmat(NaN,n,1);
C.As=repmat(NaN,n,n); 
C.iter=0; 
C.flag='';
if ~(nargin==3&&nargout<=3&&isequal(m,n)&&isequal(size(B),[n,n])&&isequal(length(b),n)&&isreal(A)&&isreal(B)&&isreal(b)&&~isintval(A)&&~isintval(B)&&~isintval(b))
    setround(gr); return                              
end
[xx,flag,AAs,iter]=eva(A,B,b);
if isequal(flag,'solution')
    x=xx;
end
if isequal(flag,'singular')&&~isempty(xx)
    y=xx;
    C.As=AAs; 
end
C.iter=iter;
C.flag=flag;
setround(gr)
%
%%%%%%%%%%%%%%%%%
% Subfunction eva
%%%%%%%%%%%%%%%%%
%
% absvalequation of 2005-11-17 renamed as eva, unchanged; 2008-04-24, unchanged 
% always gives nonempty x or As; when As is nonempty then x may be also nonempty, in this case it is an Oettli-Prager vector
% that is, in eva x plays two roles (not supported in ek)
function [x,flag,As,iter]=eva(A,B,b,ps) 
% AVE    Solving the equation A*x+B*abs(x)=b.
%
% [x,flag,As,iter]=absvalequation(A,B,b) either finds a 
% solution x to the equation 
%           A*x+B*abs(x)=b 
% (A, B square), or finds a nonzero vector x satisfying
%           abs(A*x)<=abs(B)*abs(x).
% In the latter case it also constructs a singular matrix 
% As such that As*x=0 and abs(As-A)<=abs(B). The variable
% flag gives a verbal description of the output ('solution' 
% or 'singular'), iter is the number of iterations (i.e.,
% of passes through the while loop; it may be zero if the
% sign accord is achieved immediately).

% Based on an unpublished algorithm.
%
% Copyright 2005 Jiri Rohn (rohn@cs.cas.cz)
%
if ((nargin<4)|((nargin==4)&(ps~=0)&(ps~=12))), ps=0; end  
% use [x,flag,As,iter]=absvalequation(A,B,b,12) for a paused step-by-step output
b=b(:); n=length(b); I=eye(n,n);
ep=n*(max([norm(A,inf) norm(B,inf) norm(b,inf)]))*eps;
x=[]; flag='singular'; As=[]; iter=0;
if rank(A)<n 
    As=A; x=null(As); x=x(:,1); 
    return
end
x=A\b;
for j=1:n 
    if x(j)<0, z(j)=-1; else z(j)=1; end
end 
z=z';
if rank(A+B*diag(z))<n 
    As=A+B*diag(z); x=null(As); x=x(:,1); 
    return 
end
x=(A+B*diag(z))\b; 
C=-inv(A+B*diag(z))*B;
X1=zeros(n,n);
r=zeros(1,n);
while any(z.*x<-ep)
  k=min(find(z.*x<-ep));
  iter=iter+1;
  if ps==12
      X=[z'; x'; k*I(k,:); r; iter, 1+2*z(k)*C(k,k), zeros(1,n-2)] 
      pause 
  end
  if 1+2*z(k)*C(k,k)<=0
      if rank(A+B*diag(z))==n
          C=-inv(A+B*diag(z))*B;
      end
      if 1+2*z(k)*C(k,k)<=0 
          tau=(-1)/(2*z(k)*C(k,k)); 
          As=A+B*(diag(z)-2*tau*z(k)*I(:,k)*I(k,:)); 
          if rank(As)<n  
              x=null(As); x=x(:,1); 
          else 
              x=[]; 
          end 
          return
      end
  end
  if ((k<n)&(all(r(k)>r(k+1:n))))|((k==n)&(r(k)>0))
      if rank(A+B*diag(z))==n 
          x=(A+B*diag(z))\b; 
      end
      x=x-X1(:,k);
      for j=1:n
          if x(j)<0, z(j)=-1; else z(j)=1; end
      end
      ct=A*x;
      jm=abs(B)*abs(x);
      for i=1:n
          if jm(i)>ep, y(i)=ct(i)/jm(i); else y(i)=1; end
      end
      As=A-diag(y)*abs(B)*diag(z); 
      return
  end
  X1(:,k)=x;
  r(k)=iter;
  z(k)=-z(k);
  if (mod(iter,n)==0)&(rank(A+B*diag(z))==n)
      x=(A+B*diag(z))\b; 
      C=-inv(A+B*diag(z))*B;          
  else
      alpha=2*z(k)/(1-2*z(k)*C(k,k));
      x=x+alpha*x(k)*C(:,k); 
      C=C+alpha*C(:,k)*C(k,:);
  end
end 
if rank(A+B*diag(z))==n 
    x1=(A+B*diag(z))\b;
    if ~any(z.*x1<-ep)
        x=x1;
    end
end
if ps==12 
    X=[z'; x'; zeros(1,n); r; iter, Inf, zeros(1,n-2)] 
    pause
end
flag='solution';

