function [x,As,C]=intervalhull(A,b,t) 
%INTERVALHULL     Interval hull of the solution set of a system of interval linear equations.
%
%    This is an INTLAB file. It requires to have INTLAB installed under
%    MATLAB to function properly.
%    
%    x=intervalhull(A,b) computes a verified interval hull x of the
%    solution set of a system of interval linear equations A*x=b, A square, 
%    or yields an empty vector.
%
%    [x,As]=intervalhull(A,b) gives either a verified interval hull x,
%    or a very tight ("almost thin") interval matrix As which is a part of
%    A and is verified to contain a singular matrix. If no verified result
%    is achieved, then both x and As are empty.
%
%    [x,As,C]=intervalhull(A,b) produces additional information in the
%    structure C:
%        C.xun    is an unverified interval hull, issued in case a verified
%                 one could not be computed,
%        C.Asun   is a thin unverified singular matrix, issued in case a verified 
%                 one could not be computed,
%        C.flag   describes verbally the output; it has one of the forms
%                   'verified interval hull computed              '
%                   'interval matrix verified singular            '
%                   'interval hull computed, result not verified  '
%                   'interval matrix singular, result not verified',,
%        C.orth   is the number of orthants inspected by the algorithm,
%        C.iter   is the total number of iterations of the core algorithm,
%        C.D      is a (possibly huge) matrix with rows of the form [z xl' xu'], where
%                   z is a plus/minus-one vector, and
%                   infsup(xl,xu) is a verified (possibly not optimal)
%                     enclosure of the intersection of the solution set
%                     with the z-orthant;
%                 if some plus/minus vector z is not present, then it is
%                 verified that the intersection of the solution set with
%                 the z-orthant is empty,
%        C.Xlower is a matrix such that for each i the lower bound x.inf(i) was
%                 attained as the lower bound of the solution of the system
%                   (mid(A)-diag(y)*rad(A)*diag(z))*x=mid(b)+diag(y)*rad(b),
%                 where [y z] is the ith row of C.Xlower,
%        C.Xupper is a matrix such that for each i the upper bound x.sup(i) was
%                 attained as the upper bound of the solution of the system
%                   (mid(A)-diag(y)*rad(A)*diag(z))*x=mid(b)+diag(y)*rad(b),
%                 where [y z] is the ith row of C.Xupper.
%        C.D, C.Xlower, and C.Xupper are empty if x is empty.
%
%    [x,As,C]=intervalhull(A,b,1) is the same as before, but in course of the
%                 computation it also produces a screen output of the form
%                   "Expected remaining time: ... sec."
%                 This is a useful feature, considering the NP-hardness of
%                 the problem; however, it slows down the actual computation.
%
%    See also  VERIFYLSS.

%    Copyright 2000-2007 Jiri Rohn
%
%    Based on the algorithm "hull" described in
%    J. Rohn, A Handbook of Results on Interval Linear Problems,
%    downloadable from http://www.cs.cas.cz/~rohn
%
%    WARRANTY
%
%    Because the program is licensed free of charge, there is 
%    no warranty for the program, to the extent permitted by applicable
%    law. Except when otherwise stated in writing the copyright holder
%    and/or other parties provide the program "as is" without warranty
%    of any kind, either expressed or implied, including, but not
%    limited to, the implied warranties of merchantability and fitness
%    for a particular purpose. The entire risk as to the quality and
%    performance of the program is with you. Should the program prove
%    defective, you assume the cost of all necessary servicing, repair
%    or correction.
%  

% %    OLD TEXT from 2006-02 (commented twice)
% %    [x,flag,As]=intervalhull(A,b) either computes a verified interval hull 
% %    x of the solution set of a system of interval linear equations A*x=b 
% %    (A square), or verifies singularity of the interval matrix A, or
% %    produces an output which is not verified. The input data A, b must be 
% %    given as intval objects. The easiest way to input an interval matrix 
% %    [A1,A2] and an interval vector [b1,b2] as intval objects is to type
% %    A=infsup(A1,A2), b=infsup(b1,b2) before running INTERVALHULL. The output
% %    variable flag always gives a verbal description of the output. The
% %    following four possibilities may occur:
% %              
% %    flag='verified interval hull computed              ': in this case  
% %    x is the verified interval hull and As=[]. Under verified interval hull we
% %    understand an enclosure of the interval hull which is as tight as
% %    possible: it is computed according to formulae for the interval hull,
% %    but is slightly enlarged as a result of INTLAB outward rounding in the
% %    finite precision arithmetic.
% %
% %    flag='interval hull computed, result not verified  ': in this case 
% %    x is the interval hull computed in pure MATLAB (without verification, so
% %    that the result may be afflicted with roundoff errors), and As=[].
% %    This happens when the INTLAB algorithm could not find a verified interval 
% %    hull and handed over to a MATLAB procedure which employs formulae for
% %    the interval hull, but computes without verification.
% %
% %    flag='interval matrix verified singular            ': in this case  
% %    the interval matrix A is verified to contain a singular matrix, x=[], 
% %    and As is either nonempty, or empty. Is As is nonempty, then it is a very 
% %    tight ("almost thin") interval matrix which is a subset of A and is
% %    verified to contain a singular matrix (that is, a real singular
% %    matrix, possibly not representable in finite precision arithmetic).
% %    If As is empty, then A is still verified singular, but a verified enclosure
% %    of a singular matrix has not been found by the algorithm.
% %
% %    flag='interval matrix singular, result not verified': in this case
% %    x=[] and As is a thin interval matrix computed in pure MATLAB (without
% %    verification, so that the result may be afflicted with roundoff
% %    errors). This happens when the INTLAB algorithm could not find a verified
% %    singular matrix and handed over to a MATLAB procedure which employs a formula
% %    for a singular matrix in A, but computes without verification.
% %
% %    Computational experience shows that INTERVALHULL is usually very quick
% %    in detecting singularity. Computation of a verified hull, however, is
% %    an NP-hard problem and occasionally it may last long. The procedure is
% %    based on an unpublished not-a-priori-exponential algorithm whose
% %    behavior is hardly predictable from the input data. If the computing time 
% %    is (or can be expected to be) large, using the function VERIFYLSS instead 
% %    is recommended. That procedure, however, does not compute the interval hull, 
% %    only an enclosure of it, and, in contrast to INTERVALHULL, is not fully 
% %    general (it works seemingly only under assumptions close to strong regularity).
% %
% %    The file is distributed in its preparsed pseudocode version INTERVALHULL.P.
% %
% %    Computational experience shows that verifying singularity (reg=0) is usually
% %    very fast; however, verifying regularity (reg=1) occasionally may last
% %    long since the problem is NP-hard.
% %
% %    See also  VERIFYLSS.
% 
% %    Copyright 2000-2007 Jiri Rohn
% %
% %    Based on the algorithm "hull" described in
% %    J. Rohn, A Handbook of Results on Interval Linear Problems,
% %    downloadable from http://www.cs.cas.cz/~rohn
% %
% %    NP-hardness of solving interval linear systems proved in
% %    J. Rohn and V. Kreinovich, Computing Exact Componentwise Bounds on Solutions 
% %    of Linear Systems with Interval Data is NP-Hard. SIAM Journal on Matrix 
% %    Analysis and Applications 16 (1995), 415-420.
% %
% %    WARRANTY
% %
% %    Because the program is licensed free of charge, there is 
% %    no warranty for the program, to the extent permitted by applicable
% %    law. Except when otherwise stated in writing the copyright holder
% %    and/or other parties provide the program "as is" without warranty
% %    of any kind, either expressed or implied, including, but not
% %    limited to, the implied warranties of merchantability and fitness
% %    for a particular purpose. The entire risk as to the quality and
% %    performance of the program is with you. Should the program prove
% %    defective, you assume the cost of all necessary servicing, repair
% %    or correction.
% %
% % old text from 2006-02
% %INTERVALHULL     Interval hull of the solution set of a system 
% %                 of interval linear equations.
% 
% %    Copyright 2000-2007 Jiri Rohn 
% %    Based on an unpublished algorithm. 
% %
%intvalinit('sharpivmult');
% preliminaries added 2007-10-26
% set rounding to nearest
gr=getround;
setround(0);
% setting default output
b=b(:); % added 2007-10-31
[m,n]=size(A); p=length(b);
x=[]; As=[]; C.xun=[]; C.Asun=[]; C.flag=[]; C.orth=0; C.iter=0; C.D=[]; C.Xlower=[]; C.Xupper=[];
% checking compatibility of data
if ~(m==n&&n==p&&isreal(A)&&isreal(b))
    % error('matrix not square or not real'),
    setround(gr); return
end
% handling thin data
if ~isintval(A)&&~isintval(b)
    x=verifylss(A,b);
    if ~isnan(x.inf(1))
        C.flag='verified interval hull computed              '; 
        setround(gr); return
    else
        x=[];
        C.Asun=A;
        C.flag='interval matrix singular, result not verified'; 
        setround(gr); return
    end
end
% creating full data of type intval
if ~isintval(A)
    A=infsup(A,A);                                    % allows for real input
end 
if ~isintval(b)
    b=infsup(b,b);                                    % allows for real input
end
if issparse(A)
    A=full(A);                                        % sparse matrices not implemented yet
end
% display('check')
% "t" and "time" added 2007-10-26
if (nargin==3)&&isequal(t,1)                          % t==1
    time=1;
else
    time=0;
end
[x,flag,As,C.orth,C.iter,C.D,C.Xlower,C.Xupper]=verinthull(A,b,time);
% flag='failure to produce verified output'; % for independent checking of the "inthull" part <------------
%if flag=='verified interval hull computed   '
if isequal(flag,'verified interval hull computed   ')
    C.flag='verified interval hull computed              '; 
    setround(gr); return 
end
%if flag=='interval matrix verified singular '
if isequal(flag,'interval matrix verified singular ')
    C.flag='interval matrix verified singular            '; 
    C.D=[]; C.Xlower=[]; C.Xupper=[];
    setround(gr); return 
end
%if flag=='failure to produce verified output'
if isequal(flag,'failure to produce verified output')
    x=[]; As=[]; % xun=[]; Asun=[]; flag=[]; orth=0; iter=0;
    Al=A.inf; Au=A.sup;
    bl=b.inf; bu=b.sup;
    [xl,xu,flag,C.Asun,C.orth,C.iter,C.D,C.Xlower,C.Xupper]=inthull(Al,Au,bl,bu,time);
    %if flag=='interval hull computed  '
    if isequal(flag,'interval hull computed  ')
        % x=infsup(xl,xu);
        C.xun=infsup(xl,xu); C.flag='interval hull computed, result not verified  '; 
        setround(gr); return
    end
    %if flag=='interval matrix singular'
    if isequal(flag,'interval matrix singular')
        % As=infsup(As,As);
        % Asun=As; 
        C.flag='interval matrix singular, result not verified'; 
        C.D=[]; C.Xlower=[]; C.Xupper=[];
        setround(gr); return
    end
end
%intvalinit('fastivmult');
%
%%%%% changed 2007-10 %%%%%
%%%%% the following procedures left unchanged in their states of 2006-02-09 and 2005-11-19, respectively %%%%% 
%%%%% only in inthull, subfunctions qzm, ave renamed as mzq, eva because of duplicity in verinthull %%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
% Subfunction
function [x,flag,As,orth,iter,D,Xlower,Xupper]=verinthull(A,b,time)
%VERINTHULL     Verified interval hull of the solution set 
%               of a system of interval linear equations.
%
%    This is an INTLAB file. It requires to have INTLAB installed under
%    MATLAB to function properly.
%
%    [x,flag,As]=verinthull(A,b) either computes a verified interval hull x
%    of the solution set of a system of interval linear equations A*x=b 
%    (A square), or verifies singularity of the interval matrix A, or fails
%    to produce a verified output. The input data A, b must be given as
%    intval objects. The easiest way to input an interval matrix [A1,A2]
%    and an interval vector [b1,b2] as intval objects is to type
%    A=infsup(A1,A2), b=infsup(b1,b2) before running VERINTHULL. The output
%    variable flag always gives a verbal description of the output. The
%    following possibilities may occur:
%              
%    flag='verified interval hull computed   ': in this case x is the
%    verified interval hull and As=[]. Under verified interval hull we
%    understand an enclosure of the interval hull which is as tight as
%    possible: it is computed according to formulae for the interval hull,
%    but is slightly enlarged as a result of INTLAB outward rounding in the
%    finite precision arithmetic.
%
%    flag='interval matrix verified singular ': in this case the interval
%    matrix A is verified to contain a singular matrix, x=[], and As is
%    either nonempty, or empty. Is As is nonempty, then it is a very tight
%    ("almost thin") interval matrix which is a subset of A and is
%    verified to contain a singular matrix (that is, a real singular
%    matrix, possibly not representable in finite precision arithmetic).
%    If As is empty, then A is still verified singular, but a verified enclosure
%    of a singular matrix has not been found by the algorithm.
%
%    flag='failure to produce verified output': in this case x=[], As=[],
%    and no verified output could be given. Yet there is a kind of remedy
%    in this case: try to run the pure MATLAB file INTHULL which always
%    gives an output, although not a verified one. Notice that for
%    INTHULL the interval data must be given by their bounds, not as intval
%    objects (that do not exist outside INTLAB).
%
%    Computational experience shows that VERINTHULL is usually very quick
%    in detecting singularity. Computation of a verified hull, however, is
%    an NP-hard problem and occasionally it may last long. The procedure is
%    based on an unpublished not-a-priori-exponential algorithm whose
%    behavior is hardly predictable: solving a system with a 10x10 interval
%    matrix may take essentially more time than solving a system with a 100x100 
%    interval matrix. If the computing time is (or can be expected to be) large, 
%    using the function VERIFYLSS instead is recommended. That procedure, however, 
%    does not compute the interval hull, only an enclosure of it, and, in contrast 
%    to VERINTHULL and INTHULL, is not fully general (it works seemingly only under 
%    the strong regularity assumption).

%    Copyright 2006 Jiri Rohn 
%
%    Based on an unpublished algorithm.
%
%%%% computing midpoint and radius via vermidrad added 2007-10-26
%%%% (between 2006-02-22 and 2007-10-26 unchanged)
%%%%%%%%%% main algorithm (in the form of 2005-11-19) translated to INTLAB on 2006-02-08 (old lines outcommented) %%%%%%%%%%
%%%%%%%%%% new subfunctions iniz, vqzm, vsing created on 2006-02-06 and 2006-02-07 %%%%%%%%%%
%function [xl,xu,flag,As,orth,iter]=inthull(Al,Au,bl,bu)
if time==1 % display of remaining time required
    tic, sh=1; % sh=0.5; % time shift (used in "timeprint")
    told=-sh;
end
%bl=bl(:); bu=bu(:);
b=b(:); % bl=b.inf; bu=b.sup;
% ac=0.5*(Al+Au); Delta=0.5*(Au-Al);
% ac=mid(A); Delta=rad(A);
% bc=0.5*(bl+bu); delta=0.5*(bu-bl);
% bc=mid(b); delta=rad(b);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[ac,Delta]=vermidrad(A); [bc,delta]=vermidrad(b); % added 2007-10-26
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
n=length(bc); % I=eye(n);
orth=0; iter=0; D=[];
Xlower=repmat(0,n,2*n); Xupper=repmat(0,n,2*n); % added 2007-10-31
% if rank(ac)<n, xl=[]; xu=[]; flag='interval matrix singular'; As=ac; return, end
% acinv=inv(infsup(ac,ac));
acinv=inv(ac);
if isnan(acinv.inf(1,1)) % midpoint inverse not computed
    % [flag,As]=vsing(A,ac,[]); x=[]; return % check whether ac is verified singular
    [flag,As]=vsing(A,mid(ac),[]); x=[]; return % check whether mid(ac) is verified singular
end
% xl=ac\bc; xu=xl;
x=verifylss(ac,bc); % solution of midpoint system 
if isnan(x.inf(1))  % ... not computed
    flag='failure to produce verified output';
    x=[]; As=[]; return
end
xl=x.inf; xu=x.sup; % ... computed
% z=ones(1,n); for j=1:n if xl(j)<0, z(j)=-1; end 
% 2007-11: could be now written as z=sgn(x)
z=zeros(1,n); 
for j=1:n
    if xl(j)*xu(j)>0
        if xl(j)>0, z(j)=1; else z(j)=-1; end % signs determined if equal
    end
end
B=find(xl.*xu<=0); B=B'; % set of indices for which sign of x not determined
% Z=z; 
Z=iniz(z,B); % find signs of all orthants intersected by x
D=[]; % Z: signs of orthants to be visited; D: signs of those already visited, together with bounds
while ~isempty(Z)
    p=size(Z,1); z=Z(p,:); Z=Z(1:p-1,:); % D=[D; z]; % take last, swap to D % changed 2007-10-30
    orth=orth+1; 
    %[Q,flag,As,iter1]=qzm(ac,Delta,-z');
    [Q,flag,As,iter1]=vqzm(A,-z,acinv); % computing verified Q_{-z}
    iter=iter+iter1;
    %if ~isempty(flag)
    %    xl=[]; xu=[]; flag='interval matrix singular'; return
    %end
    % if (flag=='interval matrix verified singular ')|(flag=='failure to produce verified output')
    if isequal(flag,'interval matrix verified singular ')||isequal(flag,'failure to produce verified output')
        x=[]; return
    end
    xlt=Q*bc-abs(Q)*delta; % enclosure of the lower bound of X\cap R^n_z
    for i=1:n % added 2007-10-31
        if xlt.inf(i)<xl(i)
            y=-sgn(Q(i,:)); % Q_{-z}
            Xlower(i,:)=[y' z]; % ith lower bound attained at $A_{yz}^{-1}b_y$ (unpublished)
        end
    end
    %[Q,flag,As,iter1]=qzm(ac,Delta,z');
    [Q,flag,As,iter1]=vqzm(A,z,acinv); % computing verified Q_z
    iter=iter+iter1;
    %if ~isempty(flag)
    %    xl=[]; xu=[]; flag='interval matrix singular'; return
    %end
    % if (flag=='interval matrix verified singular ')|(flag=='failure to produce verified output')
    if isequal(flag,'interval matrix verified singular ')||isequal(flag,'failure to produce verified output')
        x=[]; return
    end
    xut=Q*bc+abs(Q)*delta;    % enclosure of the upper bound of X\cap R^n_z
    for i=1:n % added 2007-10-31
        if xu(i)<xut.sup(i)
            y=sgn(Q(i,:)); % Q_z
            Xupper(i,:)=[y' z]; % ith upper bound attained at $A_{yz}^{-1}b_y$ (unpublished)
        end
    end
    xlt=xlt.inf; xut=xut.sup; % verified bounds on X\cap R^n_z
    D=[D; [z xlt' xut']];     % discharging z, keeping the bounds % changed 2007-10-30
    if all(xlt<=xut)          % X\cap R^n_z is not verified empty
        xl=min(xl,xlt);       % current global lower bound
        xu=max(xu,xut);       % current global upper bound
        for j=1:n
            zt=z; zt(j)=-zt(j);
            if (xlt(j)*xut(j)<=0)&&(~ismember(zt,Z,'rows'))&&(~ismember(zt,D(:,1:n),'rows')) % possible intersection with R^n_zt % changed 2007-10-30
                Z=[Z; zt];
            end
        end
    end
    if time==1
        told=timeprint(size(Z,1),size(D,1),sh,told);
    end
end
if time==1
    clc
    disp(['Expected remaining time: ', num2str(0), ' sec.'])
    disp(['Elapsed time: ', num2str(round(toc)), ' sec.'])
    % disp('Processing list of orthants...')
end
x=infsup(xl,xu); % verified hull
flag='verified interval hull computed   ';
As=[];
D=purifying(D);
%
%%%%%%%%%% subfunctions iniz, vqzm, qzm (changed), vsing %%%%%%%%%%
%%%%%%%%%% created and added 2006-02-06 to 2006-02-08 %%%%%%%%%%%%%
%
%%% Subfunction %%%
function Z=iniz(z,B) % initialization of Z; B is set of j's for which z_j should be \pm 1
z=z(:)';
if isempty(B), Z=z; return, end
n=length(B);
y=ones(1,n); zz=zeros(1,n);
z(B)=y;
Z=z;
while any(zz~=ones(1,n)) % algorithm for generating \pm 1-vectors
    % k=min(find(zz==0));
    k=find(zz==0,1);
    zz(1:(k-1))=zeros(1,k-1); zz(k)=1;
    y(k)=-y(k);
    z(B)=y;
    Z=[Z; z];
end
%
%%% Subfunction %%%
function [Q,flag,As,iter1]=vqzm(A,z,acinv) % verification of Q_z matrix
% ac=mid(A); Delta=rad(A);
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[ac,Delta]=vermidrad(A);       % added 2007-10-26 %
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
[Q,flag,As,iter1]=qzm(mid(ac),mid(Delta),z); % first Q_z computed and not verified
if  ~isempty(flag) % flag='singular'; Q is an Oettli-Prager candidate
    [flag,As]=vsing(A,As,Q); return % check whether As is verified singular
end
n=size(ac,1); I=eye(n,n); I=infsup(I,I);
QQ=infsup(zeros(n,n),zeros(n,n)); % QQ will be the verified Q_z
for i=1:n
    x=Q(i,:); x=x';
    y=ones(n,1);
    for j=1:n
        if x(j)<0, y(j)=-1; end % sign vector of x
    end
    % xx=verifylss(ac'-diag(z)*Delta'*diag(y),I(:,i)); % verification of ith row of Q_z % changed 2007-10-31
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    xx=absvalverifn(ac',-diag(z)*Delta',I(:,i),x); % changed 2007-10-31; two enclosures instead of one
    %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
    if isnan(xx.inf(1))
        As=mid(ac)-diag(y)*mid(Delta)*diag(z);
        if rank(As)<n
            % [flag,As]=vsing(A,As,Q); return % check whether As is verified singular
            [flag,As]=vsing(A,As,[]); return % 2007-10-26: I think it should be [] instead of Q
        else
            flag='failure to produce verified output';
            Q=[]; As=[]; return
        end
    end
%     blocked 2007-10-31: related to verification via verifylss, in absvalverifn done automatically
%     if  ~((all(y.*inf(xx)>=0))&(all(y.*sup(xx)>=0)))&~(in(acinv'*diag(z)*Delta'*abs(xx)+acinv'*I(:,i),xx)) 
%         % either xx not proved to have y as sign vector, or Rump's Brouwer test fails
%         flag='failure to produce verified output';
%         Q=[]; As=[]; return
%     end
    QQ(i,:)=xx'; % ith row of Q_z verified
end
Q=QQ; % Q_z verified
flag='                                  ';
As=[];
%
%%% Subfunction %%%
function [Q,flag,As,iter1]=qzm(ac,Delta,z) % computation of Q_z matrix, not yet verified; verified later by vqzm.m
                                           % (this is the overall aproach accepted: first ave, then qzm, only then verified vqzm)
n=size(ac,1); I=eye(n,n); iter1=0; 
Q=zeros(n,n); % preallocation: added 2007-10-26
for i=1:n
    [x,flag,As,iter]=ave(ac',-diag(z)*Delta',I(:,i),0); % computing the ith row of Q_z by solving the absolute value equation
    iter1=iter1+iter;
    % if flag=='singular'
    if isequal(flag,'singular')
        Q=x; As=As'; return % Q=[]; changed to Q=x; on 2006-02-07; Q can be used as Oettli-Prager vector in vsing.m
    end
    Q(i,:)=x';
end
flag=[]; As=[]; % Q_z computed, not yet verified
%
%%% Subfunction %%%
function [flag,As]=vsing(A,As,Q) % verification of singularity of As in frame of A
% ac=mid(A); Delta=rad(A);
%%%%%%%%%%%%%%%%%%%%%%%%%%
[ac,Delta]=vermidrad(A); % added 2007-10-26
%%%%%%%%%%%%%%%%%%%%%%%%%%
n=size(ac,1);
x1=null(As);
if (size(x1,2)==0)&&(isempty(Q)) % no null vector at disposal
    flag='failure to produce verified output'; As=[]; return
end
% if size(x1,2)>0, % blocked 2007-10-26: allow all columns of null(As) as candidates
%     x1=x1(:,1);
% end
X=[x1 Q]; % at least one candidate for an Oettli-Prager vector
for x=X
    z=ones(n,1);
    for j=1:n
        if x(j)<0, z(j)=-1; end % sign vector of x
    end
    xi=infsup(x,x);
    oeprl=abs(ac*xi);    % Oettli-Prager inequality, left  side   
    oeprr=Delta*abs(xi); % Oettli-Prager inequality, right side
    if all(oeprl.sup<=oeprr.inf) % Oettli-Prager inequality satisfied, singularity of A verified
        flag='interval matrix verified singular '; 
        y=(ac*xi)./oeprr; 
        y(find(isnan(y)))=infsup(1,1); % case od both numerator and denominator being zero
        As=ac-(diag(y)*Delta)*diag(z); % construction of singular As ...
        As=intersect(As,A);            % ... in A
        if ~any(any(isnan(As))) % intersection nowhere empty
            return % with output As
        else
            As=[]; return % with empty As, but still verified singular
        end
    end
end
flag='failure to produce verified output'; As=[]; return 
%
% ave: the core algorithm: everything evolves around it
% ave: 2007-10-26 unchanged: remains in the state of 2005-11-17
% improvement of "intlinsys" of 2007-06-24 which could not find singular matrix in some cases
%%%%%%%%%% ave: everything below this line left unchanged as of 2005-11-17 %%%%%%%%%% 
%%%%%%%%%% only comments added 2006-02 %%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
%%% Subfunction %%%
function [x,flag,As,iter]=ave(A,B,b,ps) % absvalequation of 2005-11-17 renamed as ave
% AVE    Solving the equation A*x+B*abs(x)=b.
%
% [x,flag,As,iter]=absvalequation(A,B,b) either finds a 
% solution x to the equation 
%           A*x+B*abs(x)=b 
% (A, B square), or finds a nonzero vector x satisfying
%           abs(A*x)<=abs(B)*abs(x).
% In the latter case it also constructs a singular matrix 
% As such that As*x=0 and abs(As-A)<=abs(B). The variable
% flag gives a verbal description of the output ('solution' 
% or 'singular'), iter is the number of iterations (i.e.,
% of passes through the while loop; it may be zero if the
% sign accord is achieved immediately).

% Based on an unpublished algorithm.
%
% Copyright 2005 Jiri Rohn (rohn@cs.cas.cz)
%
if ((nargin<4)|((nargin==4)&(ps~=0)&(ps~=12))), ps=0; end  
% use [x,flag,As,iter]=absvalequation(A,B,b,12) for a paused step-by-step output
b=b(:); n=length(b); I=eye(n,n);
ep=n*(max([norm(A,inf) norm(B,inf) norm(b,inf)]))*eps;
x=[]; flag='singular'; As=[]; iter=0;
if rank(A)<n % A is singular
    As=A; x=null(As); x=x(:,1); 
    return
end
x=A\b;
for j=1:n 
    if x(j)<0, z(j)=-1; else z(j)=1; end % sign vector of x
end 
z=z';
if rank(A+B*diag(z))<n % A+B*diag(z) singular
    As=A+B*diag(z); x=null(As); x=x(:,1); 
    return 
end
x=(A+B*diag(z))\b;       % first iterate
C=-inv(A+B*diag(z))*B;   % matrix used for the Sherman-Morrison updating of x
X1=zeros(n,n);           % matrix of vectors corresponding to the last appearance of k
r=zeros(1,n);            % r(k) is the iteration with last appearance of k
while any(z.*x<-ep)      % sign accord not satisfied
  k=min(find(z.*x<-ep)); % minimum index rule
  iter=iter+1;
  if ps==12 % displaying the table
      X=[z'; x'; k*I(k,:); r; iter, 1+2*z(k)*C(k,k), zeros(1,n-2)] 
      pause 
  end
  if 1+2*z(k)*C(k,k)<=0 % product of two determinants <=0
      if rank(A+B*diag(z))==n 
          C=-inv(A+B*diag(z))*B; % updating C
      end
      if 1+2*z(k)*C(k,k)<=0 % holds still with the updated value
          tau=(-1)/(2*z(k)*C(k,k)); 
          As=A+B*(diag(z)-2*tau*z(k)*I(:,k)*I(k,:)); % singular matrix in infinite precision
          if rank(As)<n % indeed singular
              x=null(As); x=x(:,1); % null vector of As
          else 
              x=[]; % singular in infinite, but not in finite precision; null vector not found
                    % this is the only case when solution of abs(A*x)<=abs(B)*abs(x) is not found in case of singularity
          end 
          return
      end
  end
  if ((k<n)&(all(r(k)>r(k+1:n))))|((k==n)&(r(k)>0)) % occurence of k leading to singularity
      if rank(A+B*diag(z))==n 
          x=(A+B*diag(z))\b; % updating
      end
      x=x-X1(:,k); % null vector satisfying the Oettli-Prager inequality (in infinite precision)
      for j=1:n
          if x(j)<0, z(j)=-1; else z(j)=1; end % sign vector
      end
      ct=A*x;
      jm=abs(B)*abs(x);
      for i=1:n
          if jm(i)>ep, y(i)=ct(i)/jm(i); else y(i)=1; end % computing y, handling zero numerator and denominator
      end
      As=A-diag(y)*abs(B)*diag(z); % singular matrix
      return
  end
  X1(:,k)=x;  % updating X1
  r(k)=iter;  % updating r
  z(k)=-z(k); % updating z
  if (mod(iter,n)==0)&(rank(A+B*diag(z))==n) % updating after each n iterations
      x=(A+B*diag(z))\b; 
      C=-inv(A+B*diag(z))*B;          
  else
      alpha=2*z(k)/(1-2*z(k)*C(k,k)); % updating
      x=x+alpha*x(k)*C(:,k);          % updating (Sherman-Morrison)
      C=C+alpha*C(:,k)*C(k,:);        % updating
  end
end 
if rank(A+B*diag(z))==n % allows updating of the result
    x1=(A+B*diag(z))\b; % updating x
    if ~any(z.*x1<-ep)  % used when sign accord achieved, otherwise the current one kept
        x=x1;
    end
end
if ps==12 % displaying the table
    X=[z'; x'; zeros(1,n); r; iter, Inf, zeros(1,n-2)] 
    pause
end
flag='solution'; % the other option is flag='singular' (both having 8 characters)
%
%%% Subfunction %%% added 2007-10-26
function [Ac,Delta]=vermidrad(A)
% computes verified midpoint and radius of A
% Ac, Delta are intval quantities
if ~isintval(A)
    Ac=infsup(A,A);
    Delta=infsup(zeros(size(A)),zeros(size(A)));
else
    Al=infsup(A.inf,A.inf);
    Au=infsup(A.sup,A.sup);
    Ac=   (Al+Au)/2;        
    Delta=(Au-Al)/2;        
end
%
%%% Subfunction %%% (common to "verinthull" and "inthull")
function told=timeprint(z,d,sh,told)
if toc-told>=sh
    clc
    % remtime=(toc/d)*z;
    % disp(['Remaining time: ', num2str(round(100*(toc/d)*z)/100), ' sec.'])
    disp(['Expected remaining time: ', num2str(round((toc/d)*z)), ' sec.'])
    told=toc;
end
%
% Subfunction % added 2007-10-30
function D=purifying(D)
[p,n]=size(D);
n=round(n/3);
ind=zeros(p,1);
% purifying of overestimations
for i=1:p 
    z= D(i,1:n);
    xl=D(i,n+1:2*n);
    xu=D(i,2*n+1:3*n);
    for j=1:n
        if z(j)==1
            xl(j)=max(xl(j),0);
        else
            xu(j)=min(xu(j),0);
        end
    end
    D(i,:)=[z xl xu];
    if all(xl<=xu)
        ind(i)=1;
    end
end
% discharging orthants with empty intersections
ind=find(ind==1);
D=D(ind,:);
% end of purifying
%
% absvalverifn of 2007-03-26; small changes 2007-09/10
function xx=absvalverifn(A,B,b,x)
%    ABSVALVERIFN     Verification of a solution of the equation A*x+B*abs(x)=b.
%
%    For an approximate real solution x of the equation
%        A*x+B*abs(x)=b,     (1)
%    xx=absvalverifn(A,B,b,x)
%    either produces a tight interval vector xx verified to contain a solution 
%    of (1), or fails (yields an interval vector xx of NaN's).
%
%    See also VERABSVALEQN, ABSVALEQUATION.

%    Copyright 2007 Jiri Rohn
%
n=size(A,1);
xx=repmat(infsup(NaN,NaN),n,1);
if ~isintval(A), A=infsup(A,A); end                   % converting data from real to intval 
if ~isintval(B), B=infsup(B,B); end
if ~isintval(b), b=infsup(b,b); end
% A, B, b assumed intval quantities already in the 2007-03-26 version
if  isintval(x), return, end
I=eye(n,n);
z=ones(n,1);
for j=1:n
    if x(j)<0, z(j)=-1; end                           % sign vector of x
end
infinf=infsup(repmat(-Inf,n,1),repmat(Inf,n,1));      % interval vector of [-Inf,Inf]'s
% first enclosure
x1=verifylss(A+B*diag(z),b);                          % enclosure via verifylss
x2=infinf;
if ~(~isnan(x1.inf(1))&&all(z.*inf(x1)>=0)&&all(z.*sup(x1)>=0)) % otherwise x1 is a verified solution
    x1=infinf;                                        % failure to produce verified output
    % second enclosure, used only if the first one fails [otherwise too time expensive]
    M=inv(I-abs(inv(A)*B));
    if isa(M,'double')                                    % bridging the current gap in verifylss
        M=infsup(M,M);
    end
    if (~isnan(M.inf(1,1)))&&(all(all(M.inf>=0)))         % M verified nonnegative
        rad=M*abs(inv(A)*(A*x+B*abs(x)-b));               % enclosure via |x^*-x|<=M*|inv(A)*residual|
        % x2=infsup(x-rad.sup,x+rad.sup);
        x2=midrad(x,rad.sup);                             % x2 is a verified solution
    else
        x2=infinf;                                        % failure to produce verified output
    end
end
% intersection of the two enclosures
x3=intersect(x1,x2);                                  
if ~isequal(x3,infinf)                                % at least one enclosure computed
    xx=x3;                                            % verified enclosure of the solution of Ax+B|x|=b
end
% end of absvalverifn
%
% Subfunction
function z=sgn(x)
% signum of x for real or intval x
% x column or row, z column
n=length(x);
z=zeros(n,1);
if ~isintval(x)
    for i=1:n
        if x(i)>=0
            z(i)=1;
        else
            z(i)=-1;
        end
    end
else % intval vector
    for i=1:n
        if x.inf(i)>0
            z(i)=1;
        end
        if x.sup(i)<0
            z(i)=-1;
        end
        % otherwise z(i)=0
    end
end
% end of sgn
%
% UNVERIFIED PART starts here
% inthull: no changes made 2007-10-26 except those rec'd by the MATLAB editor; "time" and "timeprint" added
% no changes in eva; 
% Subfunction
function [xl,xu,flag,As,orth,iter,D,Xlower,Xupper]=inthull(Al,Au,bl,bu,time)
%INTHULL     Interval hull of the solution set of a system of interval linear equations.
%
%   [xl,xu,flag]=inthull(Al,Au,bl,bu) computes the interval hull [xl,xu] of the 
%   solution set of a system of interval linear equations 
%                [Al,Au]*x=[bl,bu]
%   ([Al,Au] square), or states singularity of the interval matrix [Al,Au] 
%   in the variable flag and finds a singular matrix As in [Al,Au]; orth is
%   the number of orthants visited and iter is the number of iterations of
%   the inner algorithm.
%
%   Computation of the hull is a known NP-hard problem and it may last long.
%   Computing time crucially depends on the number of orthants intersected
%   by the solution set. If it is (or can be expected to be) large, using 
%   the function ENCLINTHULL instead is recommended. That procedure, however, 
%   does not compute the interval hull, only an enclosure of it.

%   Copyright 2005 Jiri Rohn (rohn@cs.cas.cz)
%
if time==1 
    tic, sh=1; % sh=0.5; 
    told=-sh;
end
bl=bl(:); bu=bu(:);
ac=0.5*(Al+Au); Delta=0.5*(Au-Al);
bc=0.5*(bl+bu); delta=0.5*(bu-bl);
n=length(bc); % I=eye(n);
orth=0; iter=0; D=[]; Xlower=repmat(0,n,2*n); Xupper=repmat(0,n,2*n);
if rank(ac)<n, xl=[]; xu=[]; flag='interval matrix singular'; As=ac; return, end
xl=ac\bc; xu=xl;
z=ones(1,n);
for j=1:n
    if xl(j)<0, z(j)=-1; end
end
Z=z; D=[]; 
while ~isempty(Z)
    p=size(Z,1); z=Z(p,:); Z=Z(1:p-1,:); % D=[D; z]; 
    orth=orth+1;
    [Q,flag,As,iter1]=mzq(ac,Delta,-z');
    iter=iter+iter1;
    if ~isempty(flag)
        xl=[]; xu=[]; flag='interval matrix singular'; return
    end
    xlt=Q*bc-abs(Q)*delta;
    for i=1:n
        if xlt(i)<xl(i)
            y=-sgn(Q(i,:)); % Q_{-z}
            Xlower(i,:)=[y' z];
        end
    end
    [Q,flag,As,iter1]=mzq(ac,Delta,z');
    iter=iter+iter1;
    if ~isempty(flag)
        xl=[]; xu=[]; flag='interval matrix singular'; return
    end
    xut=Q*bc+abs(Q)*delta;
    for i=1:n
        if xu(i)<xut(i)
            y=sgn(Q(i,:)); % Q_z
            Xupper(i,:)=[y' z];
        end
    end
    D=[D; [z xlt' xut']]; 
    if all(xlt<=xut)
        xl=min(xl,xlt);
        xu=max(xu,xut);
        for j=1:n
            zt=z; zt(j)=-zt(j);
            if (xlt(j)*xut(j)<=0)&&(~ismember(zt,Z,'rows'))&&(~ismember(zt,D(:,1:n),'rows'))
                Z=[Z; zt];
            end
        end
    end
    if time==1
        told=timeprint(size(Z,1),size(D,1),sh,told);
    end
end
if time==1
    clc
    disp(['Expected remaining time: ', num2str(0), ' sec.'])
    disp(['Elapsed time: ', num2str(round(toc)), ' sec.'])
    % disp('Processing list of orthants...')
end
flag='interval hull computed  ';
D=purifying(D);
%
%%% Subfunction %%%
function [Q,flag,As,iter1]=mzq(ac,Delta,z) 
n=size(ac,1); I=eye(n,n); iter1=0;
Q=zeros(n,n); % preallocation: added 2007-10-26
for i=1:n
    [x,flag,As,iter]=eva(ac',-diag(z)*Delta',I(:,i),0);
    iter1=iter1+iter;
    % if flag=='singular' % changed 2007-10-26
    if isequal(flag,'singular')
        Q=[]; As=As'; return
    end
    Q(i,:)=x';
end
flag=[]; As=[];
%
% eva: 2007-10-26 unchanged
%%% Subfunction %%%
function [x,flag,As,iter]=eva(A,B,b,ps) % absvalequation of 2005-11-17 renamed as eva
% AVE    Solving the equation A*x+B*abs(x)=b.
%
% [x,flag,As,iter]=absvalequation(A,B,b) either finds a 
% solution x to the equation 
%           A*x+B*abs(x)=b 
% (A, B square), or finds a nonzero vector x satisfying
%           abs(A*x)<=abs(B)*abs(x).
% In the latter case it also constructs a singular matrix 
% As such that As*x=0 and abs(As-A)<=abs(B). The variable
% flag gives a verbal description of the output ('solution' 
% or 'singular'), iter is the number of iterations (i.e.,
% of passes through the while loop; it may be zero if the
% sign accord is achieved immediately).

% Based on an unpublished algorithm.
%
% Copyright 2005 Jiri Rohn (rohn@cs.cas.cz)
%
if ((nargin<4)|((nargin==4)&(ps~=0)&(ps~=12))), ps=0; end  
% use [x,flag,As,iter]=absvalequation(A,B,b,12) for a paused step-by-step output
b=b(:); n=length(b); I=eye(n,n);
ep=n*(max([norm(A,inf) norm(B,inf) norm(b,inf)]))*eps;
x=[]; flag='singular'; As=[]; iter=0;
if rank(A)<n 
    As=A; x=null(As); x=x(:,1); 
    return
end
x=A\b;
for j=1:n 
    if x(j)<0, z(j)=-1; else z(j)=1; end
end 
z=z';
if rank(A+B*diag(z))<n 
    As=A+B*diag(z); x=null(As); x=x(:,1); 
    return 
end
x=(A+B*diag(z))\b; 
C=-inv(A+B*diag(z))*B;
X1=zeros(n,n);
r=zeros(1,n);
while any(z.*x<-ep)
  k=min(find(z.*x<-ep));
  iter=iter+1;
  if ps==12
      X=[z'; x'; k*I(k,:); r; iter, 1+2*z(k)*C(k,k), zeros(1,n-2)] 
      pause 
  end
  if 1+2*z(k)*C(k,k)<=0
      if rank(A+B*diag(z))==n
          C=-inv(A+B*diag(z))*B;
      end
      if 1+2*z(k)*C(k,k)<=0 
          tau=(-1)/(2*z(k)*C(k,k)); 
          As=A+B*(diag(z)-2*tau*z(k)*I(:,k)*I(k,:)); 
          if rank(As)<n  
              x=null(As); x=x(:,1); 
          else 
              x=[]; 
          end 
          return
      end
  end
  if ((k<n)&(all(r(k)>r(k+1:n))))|((k==n)&(r(k)>0))
      if rank(A+B*diag(z))==n 
          x=(A+B*diag(z))\b; 
      end
      x=x-X1(:,k);
      for j=1:n
          if x(j)<0, z(j)=-1; else z(j)=1; end
      end
      ct=A*x;
      jm=abs(B)*abs(x);
      for i=1:n
          if jm(i)>ep, y(i)=ct(i)/jm(i); else y(i)=1; end
      end
      As=A-diag(y)*abs(B)*diag(z); 
      return
  end
  X1(:,k)=x;
  r(k)=iter;
  z(k)=-z(k);
  if (mod(iter,n)==0)&(rank(A+B*diag(z))==n)
      x=(A+B*diag(z))\b; 
      C=-inv(A+B*diag(z))*B;          
  else
      alpha=2*z(k)/(1-2*z(k)*C(k,k));
      x=x+alpha*x(k)*C(:,k); 
      C=C+alpha*C(:,k)*C(k,:);
  end
end 
if rank(A+B*diag(z))==n 
    x1=(A+B*diag(z))\b;
    if ~any(z.*x1<-ep)
        x=x1;
    end
end
if ps==12 
    X=[z'; x'; zeros(1,n); r; iter, Inf, zeros(1,n-2)] 
    pause
end
flag='solution';




