function [U,S,V,E]=jk(A)
%    JK        Verified thin singular value decomposition of a complex (or real) matrix.
%
%    This is an INTLAB file. It requires to have INTLAB installed under
%    MATLAB to function properly.
%
%    For an m-by-n complex (or real) matrix A, m>=n,
%        [U,S,V,E]=jk(A)
%    computes (generally complex) m-by-n interval matrix U, a real diagonal
%    n-by-n interval matrix S and an n-by-n interval matrix V that are verified
%    to contain matrices Uo, So, Vo satisfying (in exact arithmetic):  
%        A=Uo*So*Vo',                                                         
%        Uo'*Uo=eye(n,n),
%        Vo'*Vo=eye(n,n),
%        So has nonnegative diagonal entries ordered in nonincreasing order.
%    Hence, Uo, So and Vo form a thin singular value decomposition (SVD) of A. 
%    If A is real, then U and V are real. For s=diag(S), both s.inf and s.sup 
%    are nonnegative and ordered in nonincreasing order. If no verified
%    output is given, then U, S and V consist of NaN's. 
%
%    If m<n, then the decomposition is computed by
%        [U1,S1,V1]=jk(A');
%        U=V1; S=S1'; V=U1;
%    so that U, S are m-by-m and V is n-by-m and the above properties again
%    hold, this time with
%        Uo'*Uo=eye(m,m),
%        Vo'*Vo=eye(m,m).
%
%    The structure E explains reasons for NaN output. It has three fields:
%    E.error, E.where, E.value.  
%
%    See also SVD, VERSINGVAL.

%    Copyright 2008 Jiri Rohn.
%
%    Based on the Jordan-Wielandt theorem (see G. W. Stewart and J.-G. Sun,
%    Matrix Perturbation Theory, Academic Press, San Diego 1990, p. 32).
%
%    WARRANTY
%
%    Because the program is licensed free of charge, there is 
%    no warranty for the program, to the extent permitted by applicable
%    law. Except when otherwise stated in writing the copyright holder
%    and/or other parties provide the program "as is" without warranty
%    of any kind, either expressed or implied, including, but not
%    limited to, the implied warranties of merchantability and fitness
%    for a particular purpose. The entire risk as to the quality and
%    performance of the program is with you. Should the program prove
%    defective, you assume the cost of all necessary servicing, repair
%    or correction.
%
%    History
%
%    2008-02-03   first version
%    2008-02-16   version for posting (as versvd)
%    2008-03-28   thin version, verified thin decomposition 
%    2008-03-31   output variable E added; version for posting
%    2008-04-04   transmission of error information from OL via Eol
%    2008-05-30   renamed as JK, p-coded, called by VERTHINSVD
%
gr=getround;
setround(0);
[m,n]=size(A);
if m>=n % basic case
    [Upart,Sigma,Vpart,Everthinsvdinner]=verthinsvdinner(A);
else % computed through ctranspose(A)
    [Upart1,Sigma1,Vpart1,Everthinsvdinner]=verthinsvdinner(ctranspose(A));
    Upart=Vpart1;
    Sigma=Sigma1';
    Vpart=Upart1;
end
U=Upart;
S=Sigma;
V=Vpart;
if ~isnan(U.inf(1,1)) % svd computed
    E.error='verthinsvd: none';
    E.where='NaN';
    E.value='NaN';
else % svd not computed
    E=Everthinsvdinner;
end
setround(gr);
%
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
% Subfunctions verthinsvdinner, jxj, alldisjoint
%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%%
%
function [Upart,Sigma,Vpart,E]=verthinsvdinner(A) % verthinsvd inner % changed against versvdinner % mainly in dimensions of U, S, V
% decomposes A=Upart*Sigma*ctranspose(Vpart), case m>=n, thin
gr=getround;
setround(0);
[m,n]=size(A);
Sigma=repmat(infsup(NaN,NaN),n,n); % default output
Sigma=tril(Sigma); Sigma=triu(Sigma); % diagonal matrix
Upart=repmat(infsup(NaN,NaN),m,n);
Vpart=repmat(infsup(NaN,NaN),n,n);
E.error='verthinsvdinner: none';
E.where='NaN';
E.value='NaN';
if ~(nargin==1&&nargout<=4&&m>=n&&~isintval(A))
    E.error='verthinsvdinner: wrong data';
    setround(gr); return
end
Ajw=[zeros(m,m) A; ctranspose(A) zeros(n,n)]; % Jordan-Wielandt matrix
A1=jxj(Ajw); % recasting 
[L,X,Eol]=ol(A1); % main part % ordered increasingly
if isnan(L.inf(1,1)) % not computed
    %E.error='verthinsvdinner: ol fails: eigenvalue decomposition not computed';
    E=Eol;
    setround(gr); return
end
% A1=J*Ajw*J=X*L*inv(X)=(X*J)*(J*L*J)*(J*inv(X)) implies Ajw=(J*X*J)*(J*L*J)*(J*inv(X)*J) (recasting) 
% J(m+n,1)=...=J(1,m+n)=1, 0 otherwise (permutation matrix)
L=jxj(L); % recasting to get decreasing order
for i=1:n % ensuring nonnegativity of singular values
    L(i,i)=infsup(max(L(i,i).inf,0),max(L(i,i).sup,0));
end
SSigma=L(1:n,1:n);
a=diag(SSigma);
if alldisjoint(a)==0 % singular value enclosures not mutually disjoint
    E.error='verthinsvdinner: singular value enclosures not mutually disjoint';
    setround(gr); return
end
X=jxj(X); % recasting
UUpart=X(1:m,1:n); % mxn
for j=1:n % normalizing
    alpha=norm(UUpart(:,j),2);
    if alpha.inf==0 % normalizing not possible
        E.error='verthinsvdinner: normalizing of UUpart not possible';
        E.where=['j = ',int2str(j)];
        E.value=['alpha = [',num2str(alpha.inf),', ',num2str(alpha.sup),']'];
        setround(gr); return
    end
    UUpart(:,j)=UUpart(:,j)/alpha;
end
VVpart=X(m+1:m+n,1:n); % nxn
for j=1:n % normalizing
    alpha=norm(VVpart(:,j),2);
    if alpha.inf==0 % normalizing not possible
        E.error='verthinsvdinner: normalizing of VVpart not possible';
        E.where=['j = ',int2str(j)];
        E.value=['alpha = [',num2str(alpha.inf),', ',num2str(alpha.sup),']'];
        setround(gr); return
    end
    VVpart(:,j)=VVpart(:,j)/alpha;
end
Ai=infsup(A,A);
In=infsup(eye(n,n),eye(n,n));
% checking enclosures % to be sure
if ~(all(all(in(Ai,UUpart*SSigma*ctranspose(VVpart))))&&all(all(in(In,ctranspose(UUpart)*UUpart)))&&all(all(in(In,ctranspose(VVpart)*VVpart))))
    E.error='verthinsvdinner: decomposition not valid, or some of UUpart, VVpart not orthogonal';
    setround(gr); return % enclosures not verified
end
Sigma=SSigma; % enclosures verified 
Upart=UUpart;
Vpart=VVpart;
setround(gr);
%
function Y=jxj(X)
%Y=J*X*J
% J(m+n,1)=...=J(1,m+n)=1, 0 otherwise (permutation matrix)
mn=size(X,1);
Y=X(:,mn:-1:1);
Y=Y(mn:-1:1,:);
%
function ad=alldisjoint(a)
% a an interval vector
% ad=1 if all entries of a disjoint, ad=0 otherwise
n=length(a);
for i=1:n-1
    for j=i+1:n
        if ~isnan(intersect(a(i),a(j)))
            ad=0; return
        end
    end
end
ad=1;


