function [fcr,E]=zd(A)
%    ZD      Verified full column rank of a rectangular real matrix. 
%
%    This is an INTLAB file. It requires to have INTLAB installed under
%    MATLAB to function properly.
%
%    For a rectangular real matrix A, 
%        fcr=zd(A)
%    either verifies full column rank of A, or verifies column rank
%    deficiency of A, or fails (yields no verified result): 
%
%    fcr= 1      A is verified to have full column rank (i.e., columns of A are linearly independent),
%    fcr= 0      A is verified not to have full column rank (i.e., columns of A are linearly dependent),
%    fcr=-1      no verified result.
%
%    For full row rank, apply ZD to A'.
%
%    See also PINV, VERPINV.

%    Copyright 2008 Jiri Rohn.
%
%    Full column rank is verified by showing that
%        norm(eye(size(A,2))-pinv(A)*A,p)<1
%    for p=1 or p='inf'. Column rank deficiency is proved by showing that
%    the matrix
%        B=eye(size(A,2))-pinv(A)*A
%    is verified to be nonzero.
%
%    WARRANTY
%
%    Because the program is licensed free of charge, there is 
%    no warranty for the program, to the extent permitted by applicable
%    law. Except when otherwise stated in writing the copyright holder
%    and/or other parties provide the program "as is" without warranty
%    of any kind, either expressed or implied, including, but not
%    limited to, the implied warranties of merchantability and fitness
%    for a particular purpose. The entire risk as to the quality and
%    performance of the program is with you. Should the program prove
%    defective, you assume the cost of all necessary servicing, repair
%    or correction.
%
%    History
%
%    2007-11-02   first version
%    2008-03-12   version for posting
%    2008-04-05   column rank deficiency part added; E added
%    2008-05-30   renamed as ZD, p-coded, called by VERFULLCOLRANK
%
gr=getround;
setround(0);
[m,n]=size(A);
fcr=-1;
E.error='verfullcolrank: none';
E.where='NaN';
E.value='NaN';
if ~(nargin==1&&nargout<=2&&~isintval(A))
    E.error='verfullcolrank: wrong data';
    setround(gr); return
end
I=eye(n,n);
R=pinv(A); 
R=infsup(R,R); 
G=I-R*A; % intval quantity
n1=norm(G,1);
ni=norm(G,'inf');
if min(n1.sup,ni.sup)<1 % full column rank of A verified
    fcr=1; % verified full column rank
    setround(gr); return
end
[X,Everpinv]=verpinv(A);
if isnan(X.inf(1,1))
    E=Everpinv; % no verified result
    setround(gr); return
end
% pseudoinverse computed
B=I-X*A; % B=I-pinv(A)*A % A is column rank deficient iff B is nonzero
if any(any(B.inf>0))||any(any(B.sup<0)) % B is verified nonzero
    fcr=0; % verified column rank defective
end
setround(gr);
